# Generated by Django 3.1.2 on 2021-02-01 14:40

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("blueprints", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="general",
            options={
                "default_permissions": (),
                "managed": False,
                "permissions": (
                    ("basic_access", "Can access this app"),
                    ("request_blueprints", "Can request blueprints"),
                    ("manage_requests", "Can review and accept blueprint requests"),
                    (
                        "add_personal_blueprint_owner",
                        "Can add personal blueprint owners",
                    ),
                    (
                        "add_corporate_blueprint_owner",
                        "Can add corporate blueprint owners",
                    ),
                    ("view_alliance_blueprints", "Can view alliance's blueprints"),
                    (
                        "view_blueprint_locations",
                        "Can view the location of all blueprints",
                    ),
                    (
                        "view_industry_jobs",
                        "Can view details about running industry jobs",
                    ),
                ),
            },
        ),
    ]
